@@ -47,7 +47,7 @@ name_type FT = {"false", "true"},
 
 
 SEXP String(char *V) {
-  //int i;
+  // int i;
   SEXP str;
   PROTECT(str = allocVector(STRSXP, 1)); 
   SET_STRING_ELT(str, 1, mkChar(V));
@@ -55,6 +55,19 @@ SEXP String(char *V) {
   return str;
 }
 
+SEXP String(char V[MAXUNITS][MAXCHAR], int n, int max) {
+  int i;
+  SEXP str;
+  if (V==NULL) return allocVector(VECSXP, 0);
+  if (n>max) return TooLarge(&n, 1);
+  PROTECT(str = allocVector(STRSXP, n)); 
+  for (i=0; i<n; i++) {
+    SET_STRING_ELT(str, i, mkChar(V[i]));
+  }
+  UNPROTECT(1);
+  return str;
+}
+
 
 SEXP Logi(bool* V, int n, int max) {
   int i;
@@ -255,7 +268,7 @@ SEXP Param(void* p, int nrow, int ncol, SEXPTYPE type, bool drop) {
       return String((char*) p);
     case LANGSXP : 
       {
-	const char *msg[1] = {"expression given by the user"};
+	const char *msg[1] = {"R object"};
 	return Char(msg, 1);
       }
       break; // 
@@ -375,7 +388,7 @@ SEXP GetModelInfo(cov_model *cov, int prlevel, int spConform,
 #define ninfo0 1
 #define ninfo1 6
 #define ninfo2 8
-#define ninfo3 10
+#define ninfo3 7
 #define ninfo4 3
 
   /*   !!!!!     ACHTUNG     !!!!!
@@ -569,17 +582,17 @@ SEXP GetModelInfo(cov_model *cov, int prlevel, int spConform,
     SET_STRING_ELT(nameMvec, k, mkChar("finiterange"));  
     SET_VECTOR_ELT(model, k++, ScalarLogical(cov->finiterange));
       
-    SET_STRING_ELT(nameMvec, k, mkChar("diag"));  
-    SET_VECTOR_ELT(model, k++, ScalarLogical(cov->diag));
+    //    SET_STRING_ELT(nameMvec, k, mkChar("diag"));  
+    //    SET_VECTOR_ELT(model, k++, ScalarLogical(cov->diag));
   
     //  SET_STRING_ELT(nameMvec, k, mkChar("quasidiag"));  
     //  SET_VECTOR_ELT(model, k++, ScalarLogical(cov->quasidiag));
 
-    SET_STRING_ELT(nameMvec, k, mkChar("semisep.last"));  
-    SET_VECTOR_ELT(model, k++, ScalarLogical(cov->semiseparatelast));
+    //    SET_STRING_ELT(nameMvec, k, mkChar("semisep.last"));  
+    //    SET_VECTOR_ELT(model, k++, ScalarLogical(cov->semiseparatelast));
       
-    SET_STRING_ELT(nameMvec, k, mkChar("sep.last"));  
-    SET_VECTOR_ELT(model, k++, ScalarLogical(cov->separatelast));
+    //    SET_STRING_ELT(nameMvec, k, mkChar("sep.last"));  
+    //    SET_VECTOR_ELT(model, k++, ScalarLogical(cov->separatelast));
          
     //  SET_STRING_ELT(nameMvec, k, mkChar("idx"));  
     //  SET_VECTOR_ELT(model, k++, Int(cov->idx, cov->tsdim));
@@ -589,7 +602,7 @@ SEXP GetModelInfo(cov_model *cov, int prlevel, int spConform,
 
   if (prlevel>=5) {
 
-    mpp_properties *mpp = &(cov->mpp);
+    mpp_properties *Mpp = &(cov->mpp);
     int
       vdim = cov->vdim2[0],
       nm = cov->mpp.moments,
@@ -599,15 +612,15 @@ SEXP GetModelInfo(cov_model *cov, int prlevel, int spConform,
     //  SET_VECTOR_ELT(model, k++, ScalarReal(mpp->refradius));
  
     SET_STRING_ELT(nameMvec, k, mkChar("mpp.maxheight"));  
-    SET_VECTOR_ELT(model, k++, Num(mpp->maxheights, vdim, MAX_INT));
+    SET_VECTOR_ELT(model, k++, Num(Mpp->maxheights, vdim, MAX_INT));
   
     SET_STRING_ELT(nameMvec, k, mkChar("mpp.M"));
     SET_VECTOR_ELT(model, k++, cov->mpp.moments == 0 ? R_NilValue :
-		   Num(mpp->mM, nmvdim, MAX_INT)); 
+		   Num(Mpp->mM, nmvdim, MAX_INT)); 
 
     SET_STRING_ELT(nameMvec, k, mkChar("mpp.Mplus"));
     SET_VECTOR_ELT(model, k++, cov->mpp.moments == 0 ? R_NilValue :
-		   Num(mpp->mMplus, nmvdim, MAX_INT));
+		   Num(Mpp->mMplus, nmvdim, MAX_INT));
   } 
    		 
 
@@ -763,7 +776,7 @@ int MAX_PMI = 5;
 
 void PrintPoints(location_type *loc, char *name, 
 		 double *x, coord_type xgr, long lx) {
-#ifndef RANDOMFIELDS_DEBUGGING  
+#ifndef SHOW_ADDRESSES  
   return; // unclear error below on CRAN
 #endif
 
@@ -818,7 +831,7 @@ void PrintLoc(int level, location_type *loc, bool own) {
   leer(level); PRINTF("%-10s %s\n","loc:grid", FT[loc->grid]);
   leer(level); PRINTF("%-10s %s\n","loc:dist", FT[loc->distances]);
   leer(level); PRINTF("%-10s %s\n","loc:Time", FT[loc->Time]);
-#ifdef RANDOMFIELDS_DEBUGGING  
+#ifdef SHOW_ADDRESSES  
   leer(level); PrintPoints(loc, (char *) "x", loc->x, loc->xgr, loc->lx);
   if (loc->y!=NULL || loc->ygr[0]!=NULL) {  
     // printf("not null\n");
@@ -999,8 +1012,8 @@ void pmi(cov_model *cov, char all, int level) {
 		      cov->isoprev, cov->isoown,
 		      ISONAMES[(int) cov->isoprev],
 		      ISONAMES[(int) cov->isoown]);
-  leer(level); PRINTF("%-10s %d, %d:%d, %d/%d\n","ts-x-v-dim",
-		      cov->tsdim, cov->xdimprev, cov->xdimown,
+  leer(level); PRINTF("%-10s %d, %d:%d:%d, %d/%d\n","ts-x-v-dim",
+		      cov->tsdim, cov->xdimprev, cov->xdimgatter, cov->xdimown,
 		      cov->vdim2[0], cov->vdim2[1]);  
    leer(level); PRINTF("%-10s %d\n","maxdim", cov->maxdim);  
   leer(level); PRINTF("%-10s %s (%d)\n", "type", TYPENAMES[cov->typus],
@@ -1132,7 +1145,7 @@ void pmi(cov_model *cov, char all, int level) {
     //  SHOW("pgs:delta", delta);  // fuehrt zu Fehler in valgrind ?!
     SHOWINT("pgs:nx", nx);
     if (pgs->pos != NULL) { // gauss
-#ifdef RANDOMFIELDS_DEBUGGING  
+#ifdef SHOW_ADDRESSES
       location_type *loc = Loc(cov);
       leer(level); PrintPoints(loc, (char *) "pgs.x", loc->x, pgs->xgr,loc->lx);
 #endif
@@ -1618,12 +1631,12 @@ void ple_intern(cov_fct *C){
   PRINTF("\n");
 }
 
-void ple(cov_model *cov) {
+void ple_(cov_model *cov) {
   PRINTF("  %s\n", NAME(cov));
   ple_intern(CovList + cov->nr);
 }
  
-void ple(char *name) {
+void ple_(char *name) {
   PRINTF("PLE %s\n", name);
   ple_intern(CovList + getmodelnr(name));
 }
