@@ -33,26 +33,32 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <R_ext/Lapack.h>
 #include <R_ext/Linpack.h>
 #include "RF.h"
+#include <Rdefines.h>
+
+
+// important check !!
+#ifndef LOCAL_MACHINE
+#ifdef SHOW_ADDRESSES
+SHOW_ADRESSES IS NOT ALLOWED
+#endif
+#ifdef RANDOMFIELDS_DEBUGGING
+RANDOMFIELDS_DEBUGGING IS NOT ALLOWED
+#endif
+#endif
+
 
 
 double intpow(double x, int p) {
-  //int p0 = p;
-  // double x0 = x;
-
   double res = 1.0;
   if (p < 0) {
     p = -p;
     x = 1.0 / x;
   } 
   while (p != 0) {
-    //    printf("  ... %e %d : %e\n" , x, p, res);
-  if (p % 2 == 1) res *= x;
+    if (p % 2 == 1) res *= x;
     x *= x;
     p /= 2;
   }
-
-  // printf("%e^%d = %e\n" , x0, p0, res);
-
   return res;
 }
 
@@ -80,7 +86,7 @@ void xA(double *x, double*A, int nrow, int ncol, double *y) {
     MEMCOPY(y, x, sizeof(double) * nrow);
   } else {
     for (k=i=0; i<ncol; i++) {
-      y[i] =0.0;
+      y[i] = 0.0;
       for (j=0; j<nrow; j++) {
 	y[i] += A[k++] * x[j];
       }
@@ -89,6 +95,25 @@ void xA(double *x, double*A, int nrow, int ncol, double *y) {
 }
 
 
+void xA(double *x1, double *x2,  double*A, int nrow, int ncol, double *y1,
+	double *y2) {
+  int i,j,k;
+  if (A == NULL) {
+    if (nrow != ncol || nrow <= 0) BUG;
+    MEMCOPY(y1, x1, sizeof(double) * nrow);
+    MEMCOPY(y2, x2, sizeof(double) * nrow);
+  } else {
+    for (k=i=0; i<ncol; i++) {
+      y1[i] = y2[i] = 0.0;
+      for (j=0; j<nrow; j++) {
+	y1[i] += A[k] * x1[j];
+	y2[i] += A[k++] * x2[j];
+      }
+    }
+  }	
+}
+
+
 void Ax(double *A, double*x, int nrow, int ncol, double *y) {
   int i,j,k;
   if (A == NULL) {
@@ -105,6 +130,27 @@ void Ax(double *A, double*x, int nrow, int ncol, double *y) {
 }
 
 
+void Ax(double *A, double*x1, double*x2, int nrow, int ncol, double *y1,
+	double *y2) {
+  int i,j,k;
+  if (A == NULL) {
+    if (nrow != ncol || nrow <= 0) BUG;
+    MEMCOPY(y1, x1, sizeof(double) * nrow);
+    MEMCOPY(y2, x2, sizeof(double) * nrow);
+  } else {
+    for (i=0; i<nrow; i++) y1[i]=y2[i]=0.0;
+    for (k=i=0; i<ncol; i++) { 
+      for (j=0; j<nrow; j++) {
+	y1[j] += A[k] * x1[i];
+	y2[j] += A[k++] * x2[i];
+      }
+    }
+  }
+}
+
+
+
+
 void AxResType(double *A, res_type *x, int nrow, int ncol, double *y) {
   int i,j,k;
   for (i=0; i<nrow; i++) y[i]=0.0;
@@ -229,6 +275,80 @@ double x_UxPz(double *x, double *U, double *z, int dim) {
   return xVx;
 }
 
+double getMinimalAbsEigenValue(double *Aniso, int dim) {
+  double dummy, 
+    min = RF_INF, 
+    *SICH = NULL, 
+    *D = NULL, 
+    *work = NULL;
+  int dd, Err,
+    err = NOERROR,
+    *iwork = NULL,
+     dimSq = dim * dim,
+    optim_work = 12 * dim;
+
+  if ((D =(double *) MALLOC(sizeof(double) * dim))==NULL ||
+      (work = (double *) MALLOC(sizeof(double) * optim_work))==NULL ||
+      (iwork = (int *) MALLOC(sizeof(int) * 8 * dim))==NULL ||
+      (SICH =(double *) MALLOC(sizeof(double) * dimSq))==NULL) {
+    err=ERRORMEMORYALLOCATION; goto ErrorHandling;
+  }
+  
+  MEMCOPY(SICH, Aniso, sizeof(double) * dimSq);
+  F77_CALL(dgesdd)("N", &dim, &dim, SICH, &dim, D, NULL, &dim, NULL,
+		   &dim, work, &optim_work, iwork, &Err);
+  if (Err != 0) GERR("SVD for anisotropy matrix failed.");
+  for (dd = 0; dd < dim; dd++) {
+    dummy = fabs(D[dd]);
+    if (dummy < min) min = dummy;
+  } 
+
+ ErrorHandling:
+  if (D != NULL) free(D);
+  if (SICH != NULL) free(SICH);
+  if (work != NULL) free(work);
+  if (iwork != NULL) free(iwork);
+  if (err != NOERROR) XERR(err);
+
+  return min;
+}
+
+
+double getDet(double *Aniso, int dim) {
+  double  
+    det = 1.0, 
+    *SICH = NULL, 
+    *D = NULL, 
+    *work = NULL;
+  int dd, Err,
+    err = NOERROR,
+    *iwork = NULL,
+     dimSq = dim * dim,
+    optim_work = 12 * dim;
+
+  if ((D =(double *) MALLOC(sizeof(double) * dim))==NULL ||
+      (work = (double *) MALLOC(sizeof(double) * optim_work))==NULL ||
+      (iwork = (int *) MALLOC(sizeof(int) * 8 * dim))==NULL ||
+      (SICH =(double *) MALLOC(sizeof(double) * dimSq))==NULL) {
+    err=ERRORMEMORYALLOCATION; goto ErrorHandling;
+  }
+  
+  MEMCOPY(SICH, Aniso, sizeof(double) * dimSq);
+  F77_CALL(dgesdd)("N", &dim, &dim, SICH, &dim, D, NULL, &dim, NULL,
+		   &dim, work, &optim_work, iwork, &Err);
+  if (Err != 0) GERR("SVD for anisotropy matrix failed.");
+  for (dd = 0; dd < dim; det *= D[dd++]);
+
+ ErrorHandling:
+  if (D != NULL) free(D);
+  if (SICH != NULL) free(SICH);
+  if (work != NULL) free(work);
+  if (iwork != NULL) free(iwork);
+  if (err != NOERROR) XERR(err);
+
+  return det;
+}
+
 double detU(double *C, int dim) {
   /* ACHTUNG!! detU zerstoert !!! */
   int i, info, 
@@ -251,7 +371,10 @@ void det_UpperInv(double *C, double *det, int dim) {
     dimP1 = dim + 1,
     dimsq = dim * dim;
   F77_CALL(dpofa)(C, &dim, &dim, &info); // C i s now cholesky
-  if (info != 0) ERR("det_UpperInv: dpofa failed -- is matrix positive definite?");
+  if (info != 0) {
+    //printf("C=%f %f %f %f\n", C[0], C[1], C[2], C[3]);
+    ERR("det_UpperInv: dpofa failed -- is matrix positive definite?");
+  }
 
   double Det = 1.0;
   for (i=0; i<dimsq; i+=dimP1) Det *= C[i];
@@ -340,9 +463,10 @@ void InvChol(double *C, int dim) {
 
 int invertMatrix(double *M, int size, int* Methods, int nMeth) {
   // http://www.nag.com/numeric/fl/nagdoc_fl23/xhtml/F01/f01intro.xml
-  int err, m, j, i,
+  int err, m, i,
     method = -1,
     sizeSq = size * size;
+  long j;
   double *SICH = NULL;
 
   if (nMeth > 1) {
@@ -359,8 +483,8 @@ int invertMatrix(double *M, int size, int* Methods, int nMeth) {
       memcpy(M, SICH, sizeSq);
     }
     switch(method) {
-    case 0 : // cholesky
-       F77_CALL(dpotrf)("Upper", &size, M, &size, &err);  
+    case Cholesky : // cholesky
+      F77_CALL(dpotrf)("Upper", &size, M, &size, &err);  
        if (err != 0) {
 	if (PL >= PL_COV_STRUCTURE) 
 	  PRINTF("cholesky decomposition failed; the matrix does not seem to be strictly positive definite\n");
@@ -372,7 +496,7 @@ int invertMatrix(double *M, int size, int* Methods, int nMeth) {
 	  PRINTF("Cholesky decomposition failed; matrix does not seem to be strictly positive definite\n");
 	continue;
       }
-      int  i2, i3;
+      long  i2, i3;
       for (i2=i=0; i<size; i++, i2+=size + 1) {	
 	for (i3 = i2 + 1, j = i2 + size; j<sizeSq; j+=size) M[i3++] = M[j];
       }
@@ -449,7 +573,7 @@ int invertMatrix(double *M, int size, int* Methods, int nMeth) {
       F77_CALL(dgesdd)("A",  &size,  &size, M, &size, D, U, &size, VT, &size, 
 		       work, &lwork, iwork, &err);
       
-      if (err==NOERROR && ISNA(D[0])) err=9999;
+      if (err==NOERROR && ISNAN(D[0])) err=9999;
       if (err!=NOERROR) {
 	if (PL>PL_ERRORS) PRINTF("Error code F77_CALL(dgesdd) = %d\n", err); 
 	err=ERRORDECOMPOSITION;
@@ -459,7 +583,7 @@ int invertMatrix(double *M, int size, int* Methods, int nMeth) {
       /* calculate SQRT of covariance matrix */
       for (k=0, j=0; j<size; j++) {
 	double dummy;
-	dummy = abs(D[j]) < tol ? 0.0 : 1.0 / D[j];
+	dummy = fabs(D[j]) < tol ? 0.0 : 1.0 / D[j];
 	for (i=0; i<size; i++) U[k++] *= dummy;
       }
       matmult(U, VT, M, size, size, size); // U * V
@@ -548,12 +672,17 @@ void memory_copy(void *dest, void *src, int bytes) {
 }
 
 
-void distInt(int *X, int*N, int *Genes, double *dist) {
-    int i,j, k, di, diff, *x, *y, ve, ho, endfor,
-	n = *N,
-	nP1 = n + 1,
-	genes = *Genes;
+SEXP distInt(SEXP XX, SEXP N, SEXP Genes) {
+  int i,j, k, di, diff, *x, *y, ve, ho, endfor,
+    *X = INTEGER(XX),
+    n = INTEGER(N)[0],
+    nP1 = n + 1,
+    genes = INTEGER(Genes)[0];
  
+  SEXP Dist;
+  PROTECT(Dist = allocMatrix(REALSXP, n, n));
+  double *dist = REAL(Dist);
+
   x = y = X;
   for (j=0, i=0;  j<n;  i += nP1, j++, y += genes) {
     dist[i] = 0.0;
@@ -568,13 +697,15 @@ void distInt(int *X, int*N, int *Genes, double *dist) {
       dist[ve] = dist[ho] = sqrt((double) di);
     }
   }
+  UNPROTECT(1);
+  return Dist;
 }
 
 /*
 
 SEXP GetChar(SEXP N, SEXP Choice, SEXP Shorter, SEXP Beep, SEXP Show) {  
     int i, j, 
-      start = RF_NAN,
+      start = NA_INTEGER,
       milli = 500,
       len = length(Choice), 
       n = INTEGER(N)[0],
@@ -638,9 +769,10 @@ SEXP GetChar(SEXP N, SEXP Choice, SEXP Shorter, SEXP Beep, SEXP Show) {
 */
 
 void strcopyN(char *dest, const char *src, int n) {
-  n--; assert(n > 0);
-  //  printf("%ld %ld %d\n", dest, src, n);
-  strncpy(dest, src, n);
+  if (n > 1) {
+    n--; 
+    strncpy(dest, src, n);
+  }
   dest[n] = '\0';
 }
 
@@ -708,7 +840,7 @@ double I0mL0(double x){
   double r, x2, ac;
   int i;
   
-  if (x < 0.0) {return RF_NAN;}
+  if (x < 0.0) {return RF_NA;}
   if (x < 16.0) {
     r = 0.5 * g2[0];
     ac = acos((6.0 * x - 40.0) / (x + 40.0));
@@ -737,7 +869,7 @@ double struve(double x, double nu, double factor_sign, bool expscaled)
   double sign, value, epsilon=1e-20;
   double dummy, logx, x1, x2;
   if ((x==0.0) && (nu>-1.0)) return 0.0;
-  if (x<=0) return RF_NAN; // not programmed yet
+  if (x<=0) return RF_NA; // not programmed yet
   logx = log(0.5 * x);
   x1=1.5;   
   x2=nu+1.5;   
@@ -747,7 +879,7 @@ double struve(double x, double nu, double factor_sign, bool expscaled)
     if (expscaled) dummy -= x;
     value = exp(dummy);
   } else {
-    if ( (double) ((int) (x1-0.5)) != x1-0.5 ) return RF_NAN;
+    if ( (double) ((int) (x1-0.5)) != x1-0.5 ) return RF_NA;
     value=pow(0.5 * x, nu + 1.0) / (gammafn(x1) * gammafn(x2));
     if (expscaled) value *= exp(-x);
     if ((dummy= value) <0) {
@@ -768,27 +900,36 @@ double struve(double x, double nu, double factor_sign, bool expscaled)
   return value;
 }
 
-void vectordist(double *v, int *Dim, double *Dist, int *diag){
-  int d, dim, dr;
-  double *v1, *v2, *end;
-  bool notdiag = (*diag==0);
-  dim = Dim[0];
-  end = v + Dim[1] * dim; 
+SEXP vectordist(SEXP V, SEXP DIAG){
+  bool notdiag = !LOGICAL(DIAG)[0];
+  int 
+    d, dr, 
+    rows = nrows(V), 
+    cols = ncols(V),
+    rescols = (int) (cols * (cols - 1 + 2 * (int) !notdiag) / 2);
+  double *v1, *v2, *end, *Dist,
+    *v = REAL(V);
+  
+  end = v + cols * rows; 
+  SEXP DIST;
+  
+  PROTECT(DIST = allocMatrix(REALSXP, rows, rescols));
+  Dist = REAL(DIST);
 
-//  print("%d %d %f %f\n", dim , Dim[0], v, end);
-
-  for (dr=0, v1=v; v1<end; v1+=dim) { // loop is one to large??
+  for (dr=0, v1=v; v1<end; v1+=rows) { // loop is one to large??
     v2 = v1;
     if (notdiag) {
-       v2 += dim;
+       v2 += rows;
     }
     for (; v2<end; ) {
-      for (d=0; d<dim; v2++) {
+      for (d=0; d<rows; v2++) {
 	Dist[dr++] = v1[d++] - *v2;
       }
     }
   }
-} 
+  UNPROTECT(1);
+  return DIST;
+}
 
 static int ORDERDIM;
 static double *ORDERD;
@@ -959,14 +1100,6 @@ int xMatch(char *name, char **list, unsigned int llen)  {
   } else return -1; // unmatched
 }
 
-void indextrafo(int onedimindex, int *length, int dim, int *multidimindex) {
-  int d;
-  for (d=0; d<dim; d++) {
-    multidimindex[d] = onedimindex % length[d];
-    onedimindex = onedimindex / length[d];    
-  }
-}
-
 
 int CeilIndex(double x, double *cum, int size) {  
    // der kleinste index i so das cum[i] >= x --- sollte das gleiche sein
@@ -1086,7 +1219,7 @@ double incomplete_gamma(double start, double end, double s) {
     w = 0.0;
 
   if (s <= 1.0) {
-    if (start == 0.0) return RF_NAN;
+    if (start == 0.0) return RF_NA;
   }
   
   double 
@@ -1105,8 +1238,8 @@ double incomplete_gamma(double start, double end, double s) {
     s += 1.0;
   }
   
-  w = pgamma(start, s, 1.0, 0, 0);  // q, shape, scale, lower, log
-  if (R_FINITE(end)) w -= pgamma(end, s, 1.0, 0, 0);
+  w = pgamma(start, s, 1.0, false, false);  // q, shape, scale, lower, log
+  if (R_FINITE(end)) w -= pgamma(end, s, 1.0, false, false);
 
   //  print("incomplete s=%f e=%f s=%f v=%f g=%f w=%f\n", start, end, s, v, gammafn(s), w);
 
@@ -1115,10 +1248,10 @@ double incomplete_gamma(double start, double end, double s) {
 
 
 int addressbits(void VARIABLE_IS_NOT_USED *addr) {
-#ifndef RF_DEBUGGING  
+#ifndef RANDOMFIELDS_DEBUGGING  
   return 0;
 #else
-  double x = (long int) addr,
+  double x = (intptr_t) addr,
     cut = 1e9;
   x = x - trunc(x / cut) * cut;
   return (int) x;
