@@ -72,10 +72,11 @@ void mixed(double *x, cov_model *cov, double *v) {
   int i, err, // Xnrow, 
     //    err=NOERROR,
     element = P0INT(MIXED_ELMNT),
-    vdim = cov->vdim,
+    vdim = cov->vdim2[0],
     vdimsq = vdim * vdim;
-  
-  error("mixed not programmed yet");
+  if (cov->vdim2[0] != cov->vdim2[1]) BUG;
+
+  NotProgrammedYet("");
 
  if (cov->nsub == 0) {
     for (i=0; i<vdimsq; i++) v[i] = 0.0;
@@ -93,7 +94,7 @@ void mixed(double *x, cov_model *cov, double *v) {
 
   // PMI(cov->calling);
   // crash(cov);
-  assert(false); // hier irgendwo stehen geblieben
+  NotProgrammedYet("");  // hier irgendwo stehen geblieben
 
   if (cov->q[MIXED_CONSTANT]) {
     // Xu , u ~ N(0, C)
@@ -132,7 +133,7 @@ void mixed_nonstat(double *x, double *y, cov_model *cov, double *v){
   int 
     element = P0INT(MIXED_ELMNT);
 
-  error("mixed_constant not programmed yet");
+  NotProgrammedYet("");
   
   if (element < 0 || element >= cov->nrow[MIXED_X]) { 
     // relax element <0 ??
@@ -157,7 +158,9 @@ void covmatrix_mixed(cov_model *cov, double *v) {
     CovList[sub->nr].covmatrix(sub, v);
     return;
   }
-  if (element >= cov->nrow[MIXED_X]) ERR("value of 'element' is too large");
+  if (element >= cov->nrow[MIXED_X]) {
+    ERR1("value of '%s' is too large", ELEMENT);
+  }
 
   listoftype *X = PLIST(MIXED_X);
   double *C=NULL;
@@ -170,13 +173,13 @@ void covmatrix_mixed(cov_model *cov, double *v) {
   }
   
   CovList[sub->nr].covmatrix(sub, C);
-  XCXt(X->p[element], C, v, nrow, ncol);
+  XCXt(X->p[element], C, v, nrow, ncol); BUG;//todo:?reprogramm XCXt with alloc here ?
   Loc(cov)->totalpoints = nrow;
 
   // PMI(cov);
   
   /*
-  int i,j,k, tot=Loc(cov)->totalpoints;
+    long i,j,k, tot=Loc(cov)->totalpoints;
   printf("\nStart mixed C %d  ---- check within comment\n", element); //
   for (k=i=0; i<ncol * ncol; i+=tot) {
      for (j=0; j<ncol; j++) printf("%f ", C[k++]); //
@@ -205,8 +208,9 @@ int set_mixed_constant(cov_model *cov) {
     *sub = next;   
   bool simple = true;
   location_type *loc=Loc(cov);
-  int i,
-    totalpoints = loc->totalpoints,
+  long i,
+    totalpoints = loc->totalpoints;
+  int
     *ncol = cov->ncol,
     *nrow = cov->nrow; 
   listoftype *X = PLIST(MIXED_X);
@@ -236,13 +240,13 @@ int set_mixed_constant(cov_model *cov) {
       constant_size = PARAMLIST(sub, CONSTANT_M)->nrow[i];
       
       if (ncol[MIXED_X] > 0 && X->ncol[i] != constant_size) {
-	SERR5("%dth matrix 'X' (%d x %d) and (%d x %d) constant matrix 'M' do not match", i, X->nrow[i], X->ncol[i], constant_size, constant_size);
+	SERR7("%ldth matrix '%s' (%d x %d) and (%d x %d) constant matrix '%s' do not match", i, KNAME(MIXED_X), X->nrow[i], X->ncol[i], constant_size, constant_size, NICK(sub));
       }
     }
   } else {
     for (i=0; i<nrow[MIXED_X]; i++) {
       if (X->nrow[i] != X->ncol[i])
-	SERR3("%dth  matrix is not symmetric (%d x %d)",
+	SERR3("%ldth  matrix is not symmetric (%d x %d)",
 	      i+1, X->nrow[i], X->ncol[i]);
     }
   }
@@ -251,7 +255,7 @@ int set_mixed_constant(cov_model *cov) {
     for (i=0; i<nrow[MIXED_X]; i++) {
       if (X->nrow[i] != totalpoints) {
 	//PMI(cov);
-	SERR3("number of rows of entry %d of 'X' (%d) are different from the number of locations (%d)", i+1, X->nrow[i], totalpoints);
+	SERR4("number of rows of entry %ld of '%s' (%d) are different from the number of locations (%ld)", i+1, KNAME(MIXED_X),  X->nrow[i], totalpoints);
       }
     }
   return NOERROR;
@@ -279,8 +283,7 @@ int checkmixed(cov_model *cov) {
   //location_type *loc=Loc(cov);
   int i, err, 
     nkappa = CovList[cov->nr].kappas,
-    //totalpoints = loc->totalpoints,
-    nsub = cov->nsub,
+     nsub = cov->nsub,
     *ncol = cov->ncol,
     *nrow = cov->nrow; // taken[MAX DIM],
   char msg[300];
@@ -290,7 +293,7 @@ int checkmixed(cov_model *cov) {
   for (i=0; i<nkappa; i++) // NEVER; cf kriging.R, for instance
     if (cov->kappasub[i] != NULL) SERR("parameters may not be random")
 
-  cov->vdim=1; //falls kein submodel vorhanden (Marco)
+  cov->vdim2[0] = cov->vdim2[1] = 1; //falls kein submodel vorhanden (Marco)
   cov->maxdim=INFDIM;
   cov->matrix_indep_of_x = true;
 
@@ -299,19 +302,22 @@ int checkmixed(cov_model *cov) {
   if (ncol[MIXED_BETA] > 0) { // b is given  
     if (nsub != 0) 
       SERR("b and a covariance model may not be given at the same time");
-    if (ncol[MIXED_X] == 0) SERR("if 'b' is given 'X' must be given");
+    if (ncol[MIXED_X] == 0) SERR2("if '%s' is given '%s' must be given",
+				  KNAME(MIXED_BETA), KNAME(MIXED_X));
     for (i=0; i<nrow[MIXED_X]; i++) {
       if (X->ncol[i] != nrow[MIXED_BETA]) {
 	sprintf(msg,
-       	"%dth set: (%d x %d) matrix X and (%d x %d) vector b do not match",
-	i, X->nrow[0], X->ncol[i], nrow[MIXED_BETA], ncol[MIXED_BETA]);
+		"%dth set: (%d x %d) matrix '%s' and (%d x %d) vector '%s' do not match",
+		i, X->nrow[0], X->ncol[i], KNAME(MIXED_X),
+		nrow[MIXED_BETA], ncol[MIXED_BETA], KNAME(MIXED_BETA));
 	SERR(msg);
       }
     }
   } else if (nsub == 0) { // only X is given -- then only a deterministic 
 	//                                 constant is given
     if (ncol[MIXED_BETA] == 0) 
-      SERR("if no covariance model is given then 'b' must be given");
+      SERR1("if no covariance model is given then '%s' must be given",
+	    KNAME(MIXED_BETA));
     if (ncol[MIXED_X] != 1) // deterministic effect
       SERR("X must have one column");
     kdefault(cov, MIXED_BETA, 1);
@@ -339,17 +345,19 @@ int checkmixed(cov_model *cov) {
     setbackward(cov, next);
   }
 
-  if (cov->vdim > 1) 
+  if (cov->vdim2[0] > 1) 
     SERR("multivariate version of mixed not programmed yet");
 
   if (PisNULL(MIXED_DIST) xor PisNULL(MIXED_DIM))
-    SERR("if 'dim' and 'dist' must be given at the same time");
+    SERR2("if '%s' and '%s' must be given at the same time",
+	  KNAME(MIXED_DIM), KNAME(MIXED_DIST));
   if (PisNULL(MIXED_DIST) xor PisNULL(MIXED_COORD))
-    SERR("'dist' and 'coord' may not be given together");
+    SERR2("'%s' and '%s' may not be given together", 
+	  KNAME(MIXED_DIST), KNAME(MIXED_COORD));
 
   // incorrect. but save !!
-  cov->semiseparatelast = false; // taken[tsxdim - 1] <= 1;
-  cov->separatelast = false;     // taken[tsxdim - 1] <= 1; ?? 
+  //  cov->semiseparatelast = false; // taken[tsxdim - 1] <= 1;
+  // cov->separatelast = false;     // taken[tsxdim - 1] <= 1; ?? 
   return NOERROR;
 }
 
@@ -385,7 +393,7 @@ int checkmixed(cov_model *cov) {
 }
 
 
-int initmixed(cov_model *cov, storage  VARIABLE_IS_NOT_USED *S) {
+int initmixed(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *S) {
   location_type *loc = Loc(cov);
   mixed_storage *s;
   char errorloc_save[nErrorLoc];
@@ -398,14 +406,16 @@ int initmixed(cov_model *cov, storage  VARIABLE_IS_NOT_USED *S) {
     Cdim = -1, 
     list_element=0,
     err = NOERROR,
-    vdim = cov->vdim,
-    dim = coord!=NULL ? cov->ncol[MIXED_COORD] : P0INT(MIXED_DIM),
+    vdim = cov->vdim2[0],
+    dim = coord!=NULL ? cov->ncol[MIXED_COORD] : P0INT(MIXED_DIM);
+  long
     totalpoints = loc->totalpoints,
     total = vdim * totalpoints;
 
   listoftype *X = PLIST(MIXED_X);
   bool distTF;
 
+  assert(cov->vdim2[0] == cov->vdim2[1]);
 
   return ERRORFAILED; // muss in zerlegt werden in init und struct
   // und unten struct aufrufen richtig praeparieren.
@@ -419,15 +429,11 @@ int initmixed(cov_model *cov, storage  VARIABLE_IS_NOT_USED *S) {
   sprintf(ERROR_LOC, "%s%s: ", errorloc_save, "init mixed model");
   
   assert(cov->nr = MIXEDEFFECT);
- 
 
-  // submodel exists:
-  if ((cov->Smixed = (mixed_storage*) MALLOC(sizeof(mixed_storage)))==NULL){
-      err=ERRORMEMORYALLOCATION; goto ErrorHandling;
-  }  
+  NEW_STORAGE(Smixed, MIXED, mixed_storage);
   s = cov->Smixed;
-  MIXED_NULL(s);
-  assert(false);
+
+  NotProgrammedYet("");
 
   if (s->Xb != NULL) free(s->Xb);
   if (cov->ncol[MIXED_BETA] > 0) { // b is given
@@ -444,7 +450,7 @@ int initmixed(cov_model *cov, storage  VARIABLE_IS_NOT_USED *S) {
 
   } else { // submodel is given
 
-    assert(false) ; //if (cov->q[MIXED_CONSTANT]) error("not 'constant'").
+    NotProgrammedYet(""); //if (cov->q[MIXED_CONSTANT]) error("not 'constant'").
 
     cov_model *sub, *next = cov->sub[0];
     
@@ -479,12 +485,12 @@ int initmixed(cov_model *cov, storage  VARIABLE_IS_NOT_USED *S) {
     }
     if (cov->key->nr != GAUSSPROC) addModel(&(cov->key), GAUSSPROC);   
     cov->key->calling = cov->key;
-    cov->stor = (storage *) MALLOC(sizeof(storage));
-    STORAGE_NULL(cov->stor);
+    
+    NEW_STORAGE(stor, STORAGE, gen_storage);
     if ((err = INIT(cov->key, 0, cov->stor)) != NOERROR) goto ErrorHandling;
    
     int Xnrow, Xncol;
-    Xnrow = Xncol = Cn * sub->vdim;
+    Xnrow = Xncol = Cn * sub->vdim2[0];
     if (loc->i_row==0 && loc->i_col==0) {
       if (s->mixedcov != NULL) free(s->mixedcov);   
       s->mixedcov = (double*) MALLOC(sizeof(double) * Xnrow * Xnrow);
@@ -496,25 +502,27 @@ int initmixed(cov_model *cov, storage  VARIABLE_IS_NOT_USED *S) {
 
   } // end of submodel
  
-  cov->initialised = true;
   FieldReturn(cov);
 
  ErrorHandling: 
+  cov->initialised = err == NOERROR;
   return err;
 }
 
 
 static int keeprandomeffect = false;
-void domixed(cov_model *cov, storage  VARIABLE_IS_NOT_USED *S){
+void domixed(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *S){
   location_type *loc = Loc(cov);
   mixed_storage *s = cov->Smixed;
   double *res  = cov->rf;
-  int i,
+  int 
     list_element=0,
-    vdim = cov->vdim,
+    vdim = cov->vdim2[0];
+  long i,
     totalpoints = loc->totalpoints,
     total = vdim * totalpoints;
- 
+   assert(cov->vdim2[0] == cov->vdim2[1]);
+
   listoftype 
     *X = PLIST(MIXED_X);
 
@@ -562,14 +570,16 @@ void trend(double  VARIABLE_IS_NOT_USED *x, cov_model *cov, double *v){
   // nein: model kann auch nur aus Trend bestehen!
 
   int i,
-    vSq = cov->vdim * cov->vdim;
+    vdim = cov->vdim2[0],
+    vSq = vdim * vdim;
   for (i=0; i<vSq; i++) v[i]=0.0;
   //print("trend %d %f\n", vSq, *v);
 }
 
 void trend_nonstat(double  VARIABLE_IS_NOT_USED *x, double  VARIABLE_IS_NOT_USED *y, cov_model *cov, double *v){
   int i,
-    vSq = cov->vdim * cov->vdim;
+    vdim = cov->vdim2[0],
+    vSq = vdim * vdim;
  
   if (cov->role == ROLE_COV)  for (i=0; i<vSq; i++) v[i]=0.0;
 
@@ -635,8 +645,7 @@ int checktrend(cov_model *cov){
   //    || cov->ncol[TREND_PARAM_FCT] > 0)
   //  return(ERRORNOTPROGRAMMED);
   double *mu = P(TREND_MEAN),
-    *plane = P(TREND_LINEAR),
-    *arbitraryfct = P(TREND_FCT);
+    *plane = P(TREND_LINEAR);
   int i, 
      nkappa = CovList[cov->nr].kappas,
    *polydeg = PINT(TREND_POLY),
@@ -672,17 +681,17 @@ int checktrend(cov_model *cov){
       for (i=0; i<vdim; i++) 
 	basislen += binomialcoeff(polydeg[i] + tsdim, tsdim);      
       PALLOC(TREND_PARAM_POLY, basislen, 1);
-      for(i=0; i<basislen; i++) P(TREND_PARAM_POLY)[i] = NA_REAL;
+      for(i=0; i<basislen; i++) P(TREND_PARAM_POLY)[i] = RF_NA;
     }
     cov->matrix_indep_of_x = false;
   }
   
-  if (arbitraryfct != NULL) { //brauche hier: construct.fct$vdim
+  if (!PisNULL(TREND_FCT)) { //brauche hier: construct.fct$vdim
     cov->matrix_indep_of_x = false;
 
-    error("arbitrary function not programmed yet");
+    NotProgrammedYet("arbitrary function");
 
-    assert(false);
+    
 //     kdefault(cov, TREND_PARAM_FCT, 1.0);
 //     PROTECT(envir = allocSExp(ENVSXP));
 //     SET_ENCLOS(envir, R_GlobalEnv);
@@ -709,13 +718,13 @@ int checktrend(cov_model *cov){
   }
   
   if (vdim <= 0) {
-    vdim = cov->calling->vdim;
+    vdim = cov->calling->vdim2[0];
     if (vdim <= 0) 
       SERR("multivariate dimension for trend cannot be determined.");
     PALLOC(TREND_MEAN, vdim, 1);
     for(i=0; i<vdim; i++) P(TREND_MEAN)[i] = 0.0;
   }
-  cov->vdim = vdim;
+  cov->vdim2[0] = cov->vdim2[1] = vdim;
   cov->isoown = cov->matrix_indep_of_x ? ISOTROPIC : CARTESIAN_COORD;
   
  return NOERROR;
@@ -724,6 +733,12 @@ int checktrend(cov_model *cov){
 
 
 
+int checktrendproc(cov_model *cov){
+  return checktrend(cov);
+}
+
+
+
 void rangetrend(cov_model  VARIABLE_IS_NOT_USED *cov, range_type *range){
   //P(TREND_MEAN]: mu / mean
   
@@ -744,7 +759,7 @@ void rangetrend(cov_model  VARIABLE_IS_NOT_USED *cov, range_type *range){
   
   //P(TREND_POLY]: polydeg / polynomial degree
   range->min[TREND_POLY] = 0;
-  range->max[TREND_POLY] = RF_INF;
+  range->max[TREND_POLY] = MAXINT;
   range->pmin[TREND_POLY] = 0;
   range->pmax[TREND_POLY] = 10;
   range->openmin[TREND_POLY] = false;
@@ -776,38 +791,40 @@ void rangetrend(cov_model  VARIABLE_IS_NOT_USED *cov, range_type *range){
 }
 
 				
-double GetInternalMean(cov_model *cov){
+void GetInternalMeanI(cov_model *cov, int vdim, double *mean){
+  // assuming that mean[i]=0.0 originally for all i
+  int i;
   if (cov->nr == TREND) {
     if (cov->ncol[TREND_MEAN]==1) {
-      if (cov->nrow[TREND_MEAN] != 1) {
-	return(RF_NAN); // only scalar allowed !
+      if (cov->nrow[TREND_MEAN] != vdim) {
+	for (i=0; i<vdim; i++) mean[i] = RF_NA;
+	return; // only scalar allowed !
       }
-      return P0(TREND_MEAN);
+      for (i=0; i<vdim; i++) mean[i] += P(TREND_MEAN)[i];
     } 
   }
-  double sum=0.0;
   if (cov->nr == PLUS || cov->nr == TREND) {
-    int i;
-    for (i=0; i<cov->nsub; i++)
-      sum += GetInternalMean(cov->sub[i]);
+    for (i=0; i<cov->nsub; i++) GetInternalMeanI(cov->sub[i], vdim, mean);
   }
-  return sum;
 }
 
+void GetInternalMean(cov_model *cov, int vdim, double *mean){
+  int i;
+  for (i=0; i<vdim; i++) mean[i]=0.0;
+  GetInternalMeanI(cov, vdim, mean);
+}
 
-
-int init_trend(cov_model *cov, storage *S) {
+int init_trend(cov_model *cov, gen_storage *S) {
   
   long err = NOERROR;
   trend_storage *s;
-  double
-    *arbitraryfct = P(TREND_FCT);
   int i, 
     *polydeg = PINT(TREND_POLY),
     basislen=0,
     tsdim = cov->tsdim,
-    vdim = cov->vdim;
+    vdim = cov->vdim2[0];
   //SEXP fctformals, argnames;
+  if (cov->vdim2[0] != cov->vdim2[1]) BUG;
 
   //assert(false);
 
@@ -818,13 +835,8 @@ int init_trend(cov_model *cov, storage *S) {
       basislen += binomialcoeff(polydeg[i]+tsdim,tsdim);
   }
 
-  if (cov->Strend != NULL) free(cov->Strend);
-  if ((cov->Strend = (trend_storage *) MALLOC(sizeof(trend_storage)))==NULL) {
-    err=ERRORMEMORYALLOCATION;   
-    goto ErrorHandling;
-  }
+  NEW_STORAGE(Strend, TREND, trend_storage);
   s = cov->Strend;
-  TREND_NULL(s);
 
   if ((s->x = (double *) MALLOC(sizeof(double) * tsdim))==NULL ||
       (s->xi = (int *) MALLOC(sizeof(int) * tsdim))==NULL ||
@@ -840,8 +852,8 @@ int init_trend(cov_model *cov, storage *S) {
   //the j-th column consists of the power of the j-th space-time-dimension
   
   
-  if (arbitraryfct != NULL) { //hier werden Argumente von arbitraryfct ueberprueft
-    assert(false);
+  if (!PisNULL(TREND_FCT)) { //hier werden Argumente von arbitraryfct ueberprueft
+    NotProgrammedYet("");
 //       fctformals = getAttrib(FORMALS(*((SEXP *) arbitraryfct)), R_NamesSymbol);
 //       nargs = length(fctformals);
 //       PROTECT(argnames = allocVector(STRSXP,1));
@@ -884,7 +896,7 @@ int init_trend(cov_model *cov, storage *S) {
 }
 
 
-void do_trend(cov_model *cov, storage  VARIABLE_IS_NOT_USED *s){
+void do_trend(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s){
   location_type *loc = Loc(cov);
   char errorloc_save[nErrorLoc];
   trend_storage *S = cov->Strend;
@@ -894,26 +906,30 @@ void do_trend(cov_model *cov, storage  VARIABLE_IS_NOT_USED *s){
     *mu = P(TREND_MEAN),
     *plane   = P(TREND_LINEAR),
     *polycoeff = P(TREND_PARAM_POLY),
-    *arbitraryfct = P(TREND_FCT),
     //    *fctcoeff = P(TREND_PARAM_FCT),
     **xgr = loc->xgr,
     *x = S->x,
     *evalplane = S->evalplane;
-  int i, j, k, v, w,
+
+  int  v, w, 
     basislen, startindex,
     *polydeg = PINT(TREND_POLY),
-    totalpoints = loc->totalpoints,
-    vdim = cov->vdim,
+    vdim = cov->vdim2[0],
     tsdim = cov->tsdim,
     spatialdim = loc->spatialdim,
     *len = loc->length,
-    total = totalpoints * vdim,
     *xi = S->xi,
     *powmatrix = S->powmatrix;
   //SEXP fctbody, tempres, envir, Rx;
+
+  long i, j, k,
+    totalpoints = loc->totalpoints,
+    total = totalpoints * vdim;
+ 
   res_type *res = cov->rf;
   
- 
+  assert(cov->vdim2[0] == cov->vdim2[1]);
+
   strcpy(errorloc_save, ERROR_LOC);
   sprintf(ERROR_LOC, "%s%s: ", errorloc_save, "add trend model");
  
@@ -933,7 +949,7 @@ void do_trend(cov_model *cov, storage  VARIABLE_IS_NOT_USED *s){
 	x[w]=xgr[w][XSTART];
 	xi[w]=0;
       }
-      int endfor = totalpoints * vdim;
+      long endfor = totalpoints * vdim;
       for (k=0; k<endfor; ) {
 	xA(x, plane, cov->nrow[TREND_LINEAR], cov->ncol[TREND_LINEAR],
 	   evalplane);
@@ -955,11 +971,12 @@ void do_trend(cov_model *cov, storage  VARIABLE_IS_NOT_USED *s){
 	}
       }	
     } else if (loc->Time) {
-      int m, 
-	endfor= loc->length[loc->timespacedim-1];
+      long m, 
+	endfor= loc->length[loc->timespacedim-1],
+	endfor2 = loc->spatialtotalpoints;
       for(k=m=0, t=loc->T[XSTART]; m<endfor; m++, t+=loc->T[XSTEP]) {
 	// for(t=loc->T[XSTART], i=0; t < loc->T[XEND]; t += loc->T[XSTEP]) {
-	for(m=0, j=0; m < loc->spatialtotalpoints; m++) {
+	for(m=0, j=0; m < endfor2; m++) {
 	  for(w=0; w<spatialdim; w++) x[w] = (loc->x)[j++];
 	  x[spatialdim] = t;
 	  xA(x, plane, cov->nrow[TREND_LINEAR], cov->ncol[TREND_LINEAR],
@@ -980,7 +997,7 @@ void do_trend(cov_model *cov, storage  VARIABLE_IS_NOT_USED *s){
   
   if(polydeg != NULL) {
     startindex=0;
-    int end_k = totalpoints * vdim;
+    long end_k = totalpoints * vdim;
     for(v=0; v<vdim; v++) { 
       basislen = binomialcoeff(polydeg[v] + tsdim, tsdim);
       if(isnan(polycoeff[0])) {
@@ -1012,7 +1029,8 @@ void do_trend(cov_model *cov, storage  VARIABLE_IS_NOT_USED *s){
 	}	
       } else if(loc->Time) {
 	 for(t=loc->T[XSTART], k=v; k<end_k; t+=loc->T[XSTEP]) {
-	   for(i=0, j=0; i<loc->spatialtotalpoints; i++, k+=vdim) {
+	   long endfor = loc->spatialtotalpoints;
+	   for(i=0, j=0; i<endfor; i++, k+=vdim) {
              for(w=0; w<spatialdim; w++)  x[w] = (loc->x)[j++];
 	     x[spatialdim] = t;
 	     //evaluation of trend polynomial
@@ -1031,8 +1049,8 @@ void do_trend(cov_model *cov, storage  VARIABLE_IS_NOT_USED *s){
     }
   }
   
-  if (arbitraryfct != NULL) { //muss hier arbitraryfct auswerten
-    BUG;
+  if (!PisNULL(TREND_FCT)) { //muss hier arbitraryfct auswerten
+    NotProgrammedYet("");
 //      if (isnan(fctcoeff[0])) {
 // 	ERR("Error: cannot evaluate function without coefficient.\n");
 //      }
@@ -1137,16 +1155,17 @@ int binomialcoeff(int n, int k) {
   return res;
 }
 
-void poly_basis(cov_model *cov, storage  VARIABLE_IS_NOT_USED *s) {
+void poly_basis(cov_model *cov, gen_storage  VARIABLE_IS_NOT_USED *s) {
   
   trend_storage *S = cov->Strend;
   int basislen=0, powsum, d, i, j, k, v,
     dim = cov->tsdim,
-    vdim = cov->vdim,
+    vdim = cov->vdim2[0],
     *powmatrix = S->powmatrix,
       *dimi=NULL,
       err=NOERROR;
   int *polydeg = PINT(TREND_POLY);
+  assert(cov->vdim2[0] == cov->vdim2[1]);
   
   dimi = (int *) MALLOC(dim * sizeof(int));
   if (dimi == NULL) {
@@ -1201,3 +1220,20 @@ double evalpoly(double *x, int *powmatrix, double *polycoeff, int basislen,
   return(res);
 }
 
+
+
+void likelihood_trend(double VARIABLE_IS_NOT_USED *x, double VARIABLE_IS_NOT_USED invmatrix, cov_model *cov, 
+		      double VARIABLE_IS_NOT_USED *v){ 
+  if (cov->role == ROLE_GAUSS) {
+    
+    NotProgrammedYet("");
+      //C = covarianzmatrix
+      // b = (X^T C^{-1} X)^{-1} X^top C^{-1} y
+      // Baysiean mit b ~ N(b_0, C_B) :
+      //  b = (X^T C^{-1} X + C_b^{-1})^{-1} [X^top C^{-1} y + C_b^{-1} b_0
+  } else {
+    // deterministic is OK
+    NotProgrammedYet("");
+  }
+
+}
